$(function () {
    'use strict';

    var maxNumberOfFile = 5;

    function checkIsToManyFiles(newFileLength) {
        var length = $('tbody.files').find(".progress.progress-striped.active").length;
        var isToManyFiles = (newFileLength + length > maxNumberOfFile);
        $('button[type=submit]').prop('disabled', isToManyFiles);
        $('.tips').css('visibility', isToManyFiles == true ? 'visible' : 'hidden');
    }

    $('#fileupload').fileupload({
        url: 'upload',
        type: 'POST',
        dataType: 'json',
        acceptFileTypes: /(\.|\/)(mp3|wav|aac|m4a|3gp|flac)$/i,
        change: function (e, data) {
            if (!data.files || !$.isArray(data.files)) {
                return false;
            }
            var length = data.files.length;
            checkIsToManyFiles(length);
        },
        finished: function (e, data) {
            checkIsToManyFiles(0);
        },
        processfail: function (e, data) {
            var currentFile = data.files[data.index];
            if (data.files.error && currentFile.error) {
                // there was an error, do something about it
                console.log(currentFile.error);
            }

        },
        messages: {
            maxNumberOfFiles: '超过上传文件数目限制',
            acceptFileTypes: '文件格式不正确',
            maxFileSize: '文件过大',
            minFileSize: '文件过小'
        },
        // Callback for the start of each file upload request:
        send: function (e, data) {
            var currentFile = data.files[0];
            if (data.files.error && currentFile.error) {
                // there was an error, do something about it
                console.log(currentFile.error);
                return false;
            }

            if (e.isDefaultPrevented()) {
                return false;
            }
            var that = $(this).data('blueimp-fileupload') ||
                $(this).data('fileupload');
            if (data.context && data.dataType &&
                data.dataType.substr(0, 6) === 'iframe') {
                data.context
                    .find('.progress').addClass(
                        !$.support.transition && 'progress-animated'
                    )
                    .attr('aria-valuenow', 100)
                    .children().first().css(
                        'width',
                        '100%'
                    );
            }
            return that._trigger('sent', e, data);
        },
        add: function (e, data) {
            if (e.isDefaultPrevented()) {
                return false;
            }
            if (!data.files || !$.isArray(data.files)) {
                return false;
            }

            var $this = $(this),
                that = $this.data('blueimp-fileupload') ||
                    $this.data('fileupload'),
                options = that.options;

            data.context = that._renderUpload(data.files)
                .data('data', data)
                .addClass('processing');

            options.filesContainer[
                options.prependFiles ? 'prepend' : 'append'
            ](data.context);
            that._forceReflow(data.context);
            that._transition(data.context);

            data.process(function () {
                return $this.fileupload('process', data);
            }).always(function () {
                data.context.each(function (index) {
                    $(this).find('.size').text(
                        that._formatFileSize(data.files[index].size)
                    );
                }).removeClass('processing');

                that._renderPreviews(data);
            }).done(function () {
                data.context.find('.start').prop('disabled', false);
                if ((that._trigger('added', e, data) !== false) &&
                    (options.autoUpload || data.autoUpload) &&
                    data.autoUpload !== false) {
                    data.submit();
                }
                checkIsToManyFiles(0);
            }).fail(function () {
                if (data.files.error) {
                    data.context.each(function (index) {
                        var error = data.files[index].error;
                        if (error) {
                            $(this).find('.error').text(error);

                            //Easyin
                            $(this).removeClass('processing');
                            $(this).find('.progress').remove();
                            $(this).find('.btn').remove();

                            checkIsToManyFiles(0);
                        }
                    });
                }
            });
        },

    });

});
